from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api_async
from ...utils import server_exists, server_name_2_server_id, server_id_2_server_name
from tsugu_api._typing import _ServerId, _Update
import typing


async def handler(user: User, res: MC, platform: str, channel_id: str):
    if res.args:
        server_mode = server_name_2_server_id(res.args[0])
        if not server_exists(server_mode):
            return text_response('未找到服务器，请输入正确的服务器名')
        update: _Update = {'server_mode': server_mode, }
        r = await tsugu_api_async.change_user_data(platform, user.user_id, update)
        if r:
            if r.get('status') == 'success':
                return text_response('主服务器已设置为 ' + server_id_2_server_name(server_mode))
            return text_response(r.get('data'))




import typing
from typing import List

from ..config import config
import tsugu_api
from loguru import logger
import time

from tsugu_api._typing import _ServerId


class User:
    def __init__(self, user_id: str, platform: str, server_mode: _ServerId, default_server: List[_ServerId], car: bool, server_list: list, game_ids: list, verify_code: str):
        self.user_id: str = user_id
        self.platform: str = platform
        self.server_mode: _ServerId = server_mode
        self.default_server: List[_ServerId] = default_server
        self.car: bool = car
        self.server_list: list = server_list
        self.game_ids = game_ids
        self.verify_code = verify_code


def text_response(string):
    return [{"type": "string", "string": string}]


def server_names_2_server_ids(server_name: List[str]) -> List[_ServerId]:
    return [config._s_i[code] for code in server_name]


def server_name_2_server_id(server_name: str) -> _ServerId:
    return config._s_i[server_name] if server_name in config._s_i else None


def server_ids_2_server_names(index: List[_ServerId]) -> List[str]:
    return [config._i_s[code] for code in index]


def server_id_2_server_name(index: _ServerId) -> str:
    return config._i_s[index] if index in config._i_s else None


def server_exists(server):
    if server or server == 0:
        return True
    if not server:
        return False
    return False


def get_user(user_id: str, platform: str) -> User:
    for i in range(0, config.get_remote_user_data_max_retry):
        try:
            user_data_res = tsugu_api.get_user_data(platform, user_id)
            if user_data_res.get('status') == 'failed':
                return text_response(user_data_res.get('data'))
            break
        except Exception as e:
            logger.error(f'Error: {e}')
            time.sleep(0.5)
            continue
    else:
        return text_response('网络链接异常')
    # 获取用户数据失败
    if user_data_res.get('status') == 'failed':
        return text_response(user_data_res.get('data'))
    # 构建用户对象
    user_data = user_data_res.get('data')

    if user_data.get('game_ids') is None:

        user_data['game_ids'] = []
        for i in range(0, 5):
            if user_data.get('server_list')[i]['playerId'] != 0:
                new_game_id = {"game_id": user_data.get('server_list')[i]['playerId'], "server": i}
                user_data['game_ids'].append(new_game_id)
        verify_code_all = []
        for i in range(0, 5):
            if user_data.get('server_list')[i].get('verifyCode') is not None:
                verify_code_all.append(i)
        # 有一说一，下面这行没有实际意义
        user_data['verify_code'] = '或'.join([str(user_data.get('server_list')[i].get('verifyCode')) for i in verify_code_all]) if len(verify_code_all) > 1 else verify_code_all[0] if verify_code_all else ''

    user = User(user_id=user_id,
                platform=platform,
                server_mode=user_data.get('server_mode'),
                default_server=user_data.get('default_server'),
                car=user_data.get('car'),
                server_list=user_data.get('server_list', None),
                game_ids=user_data.get('game_ids', []),
                verify_code=user_data.get('verify_code'))
    return user





from ...config import config
from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api_async
from ...utils import server_exists, server_name_2_server_id


async def handler(user: User, res: MC, platform: str, channel_id: str):
    if not res.args or len(res.args) < 2:
        return text_response('请输入正确的格式：验证绑定 你的玩家ID(数字) 服务器名(字母缩写)')
    player_id = int(res.args[0])
    server_pre = server_name_2_server_id(res.args[1])
    if not server_exists(server_pre):
        return text_response('未找到服务器，请输入正确的服务器名')
    user.server_mode = server_pre

    r = await tsugu_api_async.bind_player_verification(platform, user.user_id, user.server_mode, player_id, True)
    if r.get('status') != 'success':
        return text_response(r.get('data'))
    return text_response(f'绑定成功！现在可以使用 玩家状态 {len(user.game_ids) + 1} 查看绑定的玩家状态')






from ..command_matcher import MC
from ..config import config
from ..utils import text_response


def help_command(res: MC):
    if not res.args:
        # 读取 config.help_doc_dict 中的所有键
        command_list = list(config._help_doc_dict.keys())
        command_list.sort()
        return text_response(f'当前支持的命令有：\n{" ".join(command_list)}\n请使用"help 命令名"来查看命令的详细帮助')
    else:
        help_text = config.help_doc_dict.get(res.args[0], None)
        if help_text:
            return text_response(help_text)
        return None




from . import bind_player
from . import unbind_player
from . import player_status
from . import bind_player_verification_off
from . import bind_player_verification_on
from . import switch_car_forwarding_off
from . import switch_car_forwarding_on
from . import change_server_mode
from . import change_default_server


async def api_handler(user, res, api, platform, channel_id):
    handler = getattr(globals()[api], 'handler')
    return await handler(user, res, platform, channel_id)





from ...config import config
from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api_async
from tsugu_api_async._typing import _Update


async def handler(user: User, res: MC, platform: str, channel_id: str):
    update: _Update = {'car': True, }
    r = await tsugu_api_async.change_user_data(platform, user.user_id, update)
    return text_response('已开启车牌转发') if r.get('status') == 'success' else None





from ...config import config
from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api
from tsugu_api._typing import _Update


def handler(user: User, res: MC, platform: str, channel_id: str):
    update: _Update = {'car': False, }
    r = tsugu_api.change_user_data(platform, user.user_id, update)
    return text_response('已关闭车牌转发') if r.get('status') == 'success' else None

    # 切换服务器模式





from ..utils import server_exists, text_response, config, get_user
from ..command_matcher import match_command
from . import remote
from . import universal
from .help import help_command
from .rooms_forward import submit_rooms


def handler(message: str, user_id: str, platform: str, channel_id: str):
    # 进行 api 命令匹配
    result = universal_api_handler(user_id, message, platform, channel_id, local=False)
    if result:
        return result

    # 远程命令
    result = remote_api_handler(user_id, message, platform, channel_id)
    if result:
        return result


def universal_api_handler(user_id: str, message: str,  platform: str, channel_id: str, local=False):
    for i in config.commands:
        if res := match_command(message, commands=i['command_name']):
            api = i['api']
            user = get_user(user_id, platform)
            return universal.api_handler(user, res, api, platform, channel_id)
    # 开始匹配 help
    if res := match_command(message, commands=['help']):
        return help_command(res)
        # 开始上传车牌
    if res := match_command(message, commands=['上传车牌', '']):
        submit_rooms(res, user_id, platform)
    return None


def remote_api_handler(user_id: str, message: str,  platform: str, channel_id: str):
    for i in config.user_commands:
        if res := match_command(message, commands=i['command_name']):
            api = i['api']
            user = get_user(user_id, platform)
            return remote.api_handler(user, res, api, platform, channel_id)

    return None






from ...config import config
from ...utils import text_response, User, server_exists
from ...command_matcher import MC
import tsugu_api_async

async def handler(user: User, res: MC, platform: str, channel_id: str):
    if res.args:
        #     最后一个参数
        # if server_exists(name_2_index(res.args[-1])):
        #     user.server_mode = name_2_index(res.args[-1])
        event_id_pre: str = res.args[0]
        # if event_id_pre.isdigit():
        #     event_id = int(event_id_pre)
        #     return tsugu_api.event_stage(user.server_mode, event_id=event_id, meta=False)
        if res.text == '-m':
            return await tsugu_api_async.event_stage(user.server_mode, meta=True)

    return await tsugu_api_async.event_stage(user.server_mode, meta=False)



from ...config import config
from ...utils import text_response, User
from ...utils import server_exists , server_name_2_server_id
from ...command_matcher import MC
import tsugu_api_async


async def handler(user: User, res: MC, platform: str, channel_id: str):
    # 无参数
    if not res.args:
        return text_response('请输入档位线')

    # 一个参数
    if res.args[0].isdigit():
        tier: int = int(res.args[0])
    else:
        return text_response('请输入正确的档位线')

    if len(res.args) == 1:
        return await tsugu_api_async.lsycx(user.server_mode, tier)

    # 两个参数
    if len(res.args) == 2:
        # 两个参数都是数字
        if res.args[1].isdigit():
            if res.args[1].isdigit():
                event_id: int = int(res.args[1])
            else:
                return text_response('请输入正确的活动ID')
            return await tsugu_api_async.lsycx(user.server_mode, tier, event_id)
        elif server_exists(server_pre := server_name_2_server_id(res.args[1])):
            user.server_mode = server_pre
            return await tsugu_api_async.lsycx(user.server_mode, tier)
        else:
            return text_response('请输入正确的活动ID或服务器名称字母简写')

    if len(res.args) == 3:
        # 三个参数是 tier event_id server_mode
        if res.args[1].isdigit():
            if res.args[1].isdigit():
                event_id: int = int(res.args[1])
            else:
                return text_response('第二个参数请输入正确的活动ID')
            if server_exists(server_pre := server_name_2_server_id(res.args[2])):
                user.server_mode = server_pre
                return await tsugu_api_async.ycx(user.server_mode, tier, event_id)
            else:
                return text_response('第三个参数请输入正确的服务器名称字母简写')

    return text_response('参数过多(<=3)')




from ...config import config
from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api_async


async def handler(user: User, res: MC, platform: str, channel_id: str):
    if not res.args:
        return text_response('请输入卡面ID')
    if res.args[0].isdigit():
        card_id = int(res.args[0])
        return await tsugu_api_async.get_card_illustration(card_id)
    return text_response('请输入正确的卡面ID')



from . import get_card_illustration
from . import search_player
from . import gacha_simulate
from . import search_gacha
from . import search_event
from . import search_song
from . import song_meta
from . import search_character
from . import song_chart
from . import ycx_all
from . import ycx
from . import lsycx
from . import room_list
from . import search_card
from . import event_stage

import asyncio


async def api_handler(user, res, api, platform, channel_id):
    handler = getattr(globals()[api], 'handler')
    return await handler(user, res, platform, channel_id)




from ...config import config
from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api
from ...utils import server_exists, server_names_2_server_ids
from tsugu_api._typing import _Update


def handler(user: User, res: MC, platform: str, channel_id: str):
    default_server = server_names_2_server_ids(res.args)
    if not default_server:
        return text_response('未找到服务器，请输入正确的服务器名')
    change_data: _Update = {'default_server': default_server}
    r = tsugu_api.change_user_data(platform, user.user_id, change_data)
    if r.get('status') == 'success':
        return text_response(f'默认服务器已设置为 {", ".join(res.args)}')
    return text_response(r.get('data'))


from ...config import config
from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api
from ...utils import server_exists, server_name_2_server_id


def handler(user: User, res: MC, platform: str, channel_id: str):
    if not res.args or len(res.args) > 1:
        return text_response('请输入正确的格式：验证解绑 记录编号(数字)')
    if not res.args[0].isdigit():
        return text_response('请输入正确的格式：验证解绑 记录编号(数字)')
    if int(res.text) <= 0:
        return text_response('请输入正确的格式：验证解绑 记录编号(数字)，编号从1开始')
    if len(user.game_ids) < int(res.text):
        return text_response('未找到记录')
    player_id = user.game_ids[int(res.args[0]) - 1].get("game_id")
    server_mode = user.game_ids[int(res.args[0]) - 1].get("server")
    r = tsugu_api.bind_player_verification(platform, user.user_id, server_mode, player_id, False)
    if r.get('status') != 'success':
        return text_response(r.get('data'))
    return text_response('解绑成功！')





from ...utils import text_response, User
from ...command_matcher import MC
import tsugu_api_async
from ...utils import server_exists, server_name_2_server_id, server_id_2_server_name


def mask_data(game_id: str):
    game_id = str(game_id)
    if len(game_id) < 6:
        return game_id[:3] + '*' * (len(game_id) - 3)
    elif len(game_id) < 3:
        return '*' * len(game_id)
    else:
        game_id = game_id[:3] + '*' * (len(game_id) - 6) + game_id[-3:]
    return game_id


async def handler(user: User, res: MC, platform: str, channel_id: str):
    bind_record = '\n'.join(
        [f'{i + 1}. {mask_data(x.get("game_id"))} {server_id_2_server_name(x.get("server"))}' for i, x in
         enumerate(user.game_ids)])
    if not res.args:
        return text_response(f'请输入正确的记录(数字)\n例如: 解除绑定 1\n当前的绑定记录如下:\n{bind_record}')

    if not res.args[0].isdigit():
        return text_response(f'请输入正确的记录(数字)\n例如: 解除绑定 1\n当前的绑定记录如下:\n{bind_record}')

    if int(res.args[0]) > len(user.game_ids):
        return text_response(f'未找到记录 {res.args[0]}，当前的绑定记录如下:\n{bind_record}')

    server_mode = user.game_ids[int(res.args[0]) - 1].get("server")
    r = await tsugu_api_async.bind_player_request(platform, user.user_id, server_mode, False)
    if r.get('status') != 'success':
        return text_response(r.get('data'))
    # 如果是200
    return text_response(
        f'''正在解除，请将 评论(个性签名) 或者 当前使用的 乐队编队名称改为\n{r.get('data')['verifyCode']}\n稍等片刻等待同步后，发送\n验证解绑 {res.args[0]}\n来完成本次身份验证''')





import tsugu_api_async

from ..utils import config
import re
from ..utils import User, text_response
from ..command_matcher import MC
from loguru import logger
from ..utils import get_user

async def submit_rooms(res: MC, user_id, platform=None):
    message = res.text
    # 检查car_config['car']中的关键字
    for keyword in config._car_config["car"]:
        if str(keyword) in message:
            break
    else:
        return []

    # 检查car_config['fake']中的关键字
    for keyword in config._car_config["fake"]:
        if str(keyword) in message:
            return []

    pattern = r"^\d{5}(\D|$)|^\d{6}(\D|$)"
    if not re.match(pattern, message):
        return []

    user = await get_user(user_id, platform)

    # 获取用户数据
    try:
        if platform:
            if not user.car:
                return []
    except Exception as e:
        # 默认不开启关闭车牌，继续提交
        pass

    try:
        car_id = message[:6]
        if not car_id.isdigit() and car_id[:5].isdigit():
            car_id = car_id[:5]

        await tsugu_api_async.submit_room_number(int(car_id), user.user_id, message, config.token_name, config.bandori_station_token)
        return []
    except Exception as e:
        logger.error(f"[Tsugu] 发生异常: {e}")
        return []  # 虽然提交失败，但是确定了是车牌消息



